----------------------------------------
--                                    --
-- @name: 	Startup script            --
-- @author: SWAT                      --
-- @url:	http://www.dc-swat.ru     --
--                                    --
----------------------------------------
--
-- Internal DreamShell lua functions:
-- 
--	system 				- DreamShell command line. Deprecated, using os.execute
--	OpenModule			- Open module and return ID
--	CloseModule			- Close module by ID
--	GetModuleByName		- Get module ID by module NAME
--	OpenApp				- Open app by name
--	GetWorkPath			- Return current DreamShell work directory. Deprecated, using os.getenv("PATH")
--	SetWorkPath			- Set current DreamShell work directory. Deprecated, using os.setenv
--	ShowConsole
--	HideConsole
--	SetDebugIO			- Set the debug output (scif, dclsocket, fb, ds, sd). By default is ds.
--
--	Bit library: 			bit.or, bit.and, bit.not, bit.xor
--	File system library: 	lfs.chdir, lfs.currentdir, lfs.dir, lfs.mkdir, lfs.rmdir
--	SPI library: 			spi.delay
--	


local DreamShell = {

	initialized = false,
	
	modules = {
		--"tolua",
		--"tolua++",
		--"luaDS",
		--"luaKOS",
		--"luaSDL",
		--"luaGUI",
		--"luaMXML",
		--"luaSTD",
		--"sqlite3",
		--"luaSQL",			-- Depends of the module sqlite3
		--"angelscript",
		--"zip",
		--"zipfs",			-- Depends of the module zip
		--"bzip2",
		--"minilzo",
		--"dcload",
		--"DSN",
		--"lwip",
		--"http",			-- Depends of the module lwip
		--"snddrv",
		--"mpglib",
		--"mpg123",
		--"oggvorbis",
		--"adx",
		--"s3m",
		--"xvid",
		--"SDL_mixer",		-- Depends of the module oggvorbis
		--"ffmpeg",			-- Depends of the modules oggvorbis and mpg123
		--"vcdc",
		--"kglx",
		--"home",			-- Depends of the module kglx
		--"isofs",			-- Depends of the module minilzo
		--"isoldr",			-- Depends of the module isofs
		"vkb"
	},

	Initialize = function(self)
	
		-- If you use emulator, set the debug i/o to scif.
		--SetDebugIO("scif");
		--SetDebugIO("dclsocket");
		
		-- Change video mode if needed. Type "vmode -l" for print list of modes.
		--os.execute("vmode -m -d 4 -p 1");
	
		local path = os.getenv("PATH");

		print(os.getenv("HOST") .. " " .. os.getenv("VERSION") .. "\n");
		print(os.getenv("ARCH") .. ": " .. os.getenv("BOARD_ID") .. "\n");
		print("Date: " .. os.date() .. "\n");
		print("Base path: " .. path .. "\n\n");

		table.foreach(self.modules, function(k, name)  
			print("DS_PROCESS: Loading module " .. name .. "...\n");
			if not OpenModule(path .. "/modules/"..name..".klf") then
				print("DS_ERROR: Can't load module " .. path .. "/modules/" .. name .. ".klf\n");
			end
		end);
		
		self:CheckUpdates(path);

		-- Initializing dc-load fs and try connect to host
		--if os.execute("dcload -i -p /lan -h 192.168.0.103:31313") == 0 then
			--self:CheckUpdates("/lan");
		--end

		-- Connecting to DreamShell Network
		--if os.execute("dsn -c -h 188.40.21.29 -l DreamShell -p DreamShell") == 0 then
			--self:CheckUpdates("/dsn");
		--end

		self:InstallingApps(path .. "/apps");
		
		if os.execute("app -o -n Main") ~= 0 then
			print("DS_ERROR: Can't open Main app\n");
			ShowConsole();
		end
		
		self.initialized = true;
	end,
	
	CheckUpdates = function(self, path)
	
		print("DS_PROCESS: Checking for updates on " .. path .. " ...\n");
	
		local f = io.open(path.. "/update/version.dat");
		local v = os.getenv("$VERSION");
		
		local ins = false;
		local update = "";
		local uf = nil;
		
		for ver in f:lines() do 

			if ins then
			
				update = path .. "/update/pkg/" .. ver .. ".dsi";
				uf = io.open(update);
				print("DS_PROCESS: Installing update ".. update .. "\n");
				
				if uf ~= nil then
					uf:close();
					os.execute("install " .. update);
				end
			end
			
			if ver == v then 
				ins = true;
			end
		end
		
		f:close();
	end,

	InstallingApps = function(self, path)

		print("DS_PROCESS: Installing apps...\n");
		
		for ent in lfs.dir(path) do
			if ent ~= nil and ent.name ~= ".." and ent.name ~= "." and ent.name ~= ".svn" and ent.size < 0 then
				os.execute(string.format("app -a -f %s/%s/app.xml", path, ent.name));
			end
		end
		
		return true;
	end
};

if not DreamShell.initialized then
	DreamShell:Initialize();
end
